# $Id: Tool.py,v 1.8 2005/12/22 06:29:08 asc Exp $

__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/ImageWidgetSpeechTool/"
__version__    = "1.0"
__cvsversion__ = "$Revision: 1.6 $"
__date__       = "$Date: 2005/12/29 17:08:40 $"
__copyright__  = "Copyright (c) 2005-2006 Aaron Straup Cope. GNU GPL 2"

import os
import codecs
import sys
import wx
import tempfile
import thread
import string
from time import sleep

from ImageWidgetSpeechTool.Constants import *

class SpeechTool :

    def Setup (self, frame) :

        self.speech_lock = thread.allocate_lock()
        self.speech_pid  = None

        self.valid_keys = ["S"]
        
        self.__lastspoken = [];
        
        SpeechMenu = wx.Menu()

        SpeechMenu.Append(SPEECH_ID_SPEAK,
                          SPEECH_MENU_SPEAK,
                          SPEECH_DESCRIBE_SPEAK)

        SpeechMenu.Append(SPEECH_ID_SHUTUP,
                          SPEECH_MENU_SHUTUP,
                          SPEECH_DESCRIBE_SHUTUP)

        wx.EVT_MENU(frame,
                    SPEECH_ID_SPEAK,
                    self.SpeakPhoto)

        wx.EVT_MENU(frame,
                    SPEECH_ID_SHUTUP,
                    self.ShutUp)

        menu = frame.GetMenuBar()
        menu.Append(SpeechMenu, SPEECH_MENU_LABEL)

        if sys.platform!='darwin' :
            menu.Enable(SPEECH_ID_SPEAK, False)

        menu.Enable(SPEECH_ID_SHUTUP, False)

        #
        
        self.Bind(wx.EVT_END_PROCESS, self.OnSpoken)  

    #
    #
    #

    def SpeechKeyDown (self, key, event=None) :

        if not key in self.valid_keys :
            return False

        if event == None :
            return True

        if not event.CmdDown() :
            return False

        if key == "S" :
            if event.ShiftDown() :
                self.SpeakPhoto(event)
            elif event.AltDown() :
                self.ShutUp(event)
            else :
                return False
        else :
            return False
        
    #
    #
    #
    
    def SpeakPhoto (self, event) :

	try :
	    photo = self.GetCurrentPhoto()
	except Exception, e:
	    self.ReportError(e)
	    return False

        #
        
        self.speech_lock.acquire()
        is_speaking = self.speech_pid
        self.speech_lock.release()

        if is_speaking == None :
            txt = u"%s. %s" % (photo['title'],photo['description'])
            self.__speak(txt)
            return True

        #
        
        return False

    #
    #
    #

    def ShutUp (self,evt) :
        
        if self.speech_pid :
            wx.Kill(self.speech_pid,wx.SIGQUIT)
            self.SetSpeakingPID(None)
            self.ToggleMenuOptions(True,False)
            
    #
    #
    #

    def FinishSpeaking (self) :
        spid = self.SpeakingPID()

        if not spid == None :
            try :
                os.waitpid(spid,0)
            except Exception, e :
                pass

    #
    #
    #
    
    def SpeakingPID (self) :
        self.speech_lock.acquire()
        pid = self.speech_pid
        self.speech_lock.release()
        return pid
    
    #
    #
    #
    
    def SetSpeakingPID (self,pid) :
        self.speech_lock.acquire()
        self.speech_pid = pid
        self.speech_lock.release()

    #
    #
    #
    
    def ToggleMenuOptions (self,speak,shutup) :
        menu = self.GetTopWindow().GetMenuBar()
        menu.Enable(SPEECH_ID_SPEAK, speak)
        menu.Enable(SPEECH_ID_SHUTUP, shutup)

    #
    #
    #
    
    def __speak (self, txt) :
              
        self.__purge()

        #
        
        platform=sys.platform
        
        if platform=='darwin' :
            self.__speak_darwin(txt)
            return True
        else :

            # For future reference :
            # import win32api
            # win32api.TerminateProcess(pid,0)

            self.ReportError("%s is not a supported platform for the speech tool" % platform)
            return False

    #
    #
    #
    
    def __speak_darwin (self,txt) :

        (fd,fname) = tempfile.mkstemp()
        fh         = codecs.open(fname, "w", "utf-8")
        
        fh.write(txt)
        fh.close()
        
        self.process = wx.Process(self)
        
        cmd = "/usr/bin/say -v Fred -f %s" % fname
        pid = wx.Execute(cmd,wx.EXEC_ASYNC,self.process)

        self.__lastspoken.append(fname)
        self.ToggleMenuOptions(False,True)
        self.SetSpeakingPID(pid)

    #
    #
    #

    def OnDisplay (self) :
        self.FinishSpeaking()
    
    def OnNext (self, event) :
        self.ShutUp(event)

    def OnPause (self, event) :
        self.ShutUp(event)
    
    def OnQuit (self, event) :
        self.ShutUp(event)
        
    #
    #
    #

    def OnSpoken (self, evt) :
        self.ToggleMenuOptions(True,False)                
        self.SetSpeakingPID(None)
        return True
    
    #
    #
    #

    def __purge (self) :
        for f in self.__lastspoken :
            if os.path.isfile(f) :
                os.unlink(f)
            self.__lastspoken.remove(f)

    #
    #
    #
